'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var PropTypes = _interopDefault(require('prop-types'));
var React = require('react');
var React__default = _interopDefault(React);

function _typeof(obj) {
  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  }
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance");
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function _toPrimitive(input, hint) {
  if (typeof input !== "object" || input === null) return input;
  var prim = input[Symbol.toPrimitive];

  if (prim !== undefined) {
    var res = prim.call(input, hint || "default");
    if (typeof res !== "object") return res;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }

  return (hint === "string" ? String : Number)(input);
}

function _toPropertyKey(arg) {
  var key = _toPrimitive(arg, "string");

  return typeof key === "symbol" ? key : String(key);
}

function getBy(obj, path, def) {
  if (!path) {
    return obj;
  }

  var pathObj = makePathArray(path);
  var val;

  try {
    val = pathObj.reduce(function (cursor, pathPart) {
      return cursor[pathPart];
    }, obj);
  } catch (e) {// continue regardless of error
  }

  return typeof val !== "undefined" ? val : def;
}
function defaultOrderByFn(arr, funcs, dirs) {
  return _toConsumableArray(arr).sort(function (rowA, rowB) {
    for (var i = 0; i < funcs.length; i += 1) {
      var sortFn = funcs[i];
      var desc = dirs[i] === false || dirs[i] === "desc";
      var sortInt = sortFn(rowA, rowB);

      if (sortInt !== 0) {
        return desc ? -sortInt : sortInt;
      }
    }

    return dirs[0] ? rowA.index - rowB.index : rowB.index - rowA.index;
  });
}
function defaultSortByFn(a, b, desc) {
  // force null and undefined to the bottom
  a = a === null || a === undefined ? "" : a;
  b = b === null || b === undefined ? "" : b; // force any string values to lowercase

  a = typeof a === "string" ? a.toLowerCase() : a;
  b = typeof b === "string" ? b.toLowerCase() : b; // Return either 1 or -1 to indicate a sort priority

  if (a > b) {
    return 1;
  }

  if (a < b) {
    return -1;
  } // returning 0, undefined or any falsey value will defer to the next
  // sorting mechanism or eventually the columns index via the orderByFn


  return 0;
}
function getFirstDefined() {
  for (var i = 0; i < arguments.length; i += 1) {
    if (typeof (i < 0 || arguments.length <= i ? undefined : arguments[i]) !== "undefined") {
      return i < 0 || arguments.length <= i ? undefined : arguments[i];
    }
  }
}
function defaultGroupByFn(rows, grouper) {
  return rows.reduce(function (prev, row, i) {
    var resKey = typeof grouper === "function" ? grouper(row.values, i) : row.values[grouper];
    prev[resKey] = Array.isArray(prev[resKey]) ? prev[resKey] : [];
    prev[resKey].push(row);
    return prev;
  }, {});
}
function defaultFilterFn(row, id, value, column) {
  return row.values[id] !== undefined ? String(row.values[id]).toLowerCase().includes(String(value).toLowerCase()) : true;
}
function setBy() {
  var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var path = arguments.length > 1 ? arguments[1] : undefined;
  var value = arguments.length > 2 ? arguments[2] : undefined;

  var recurse = function recurse(obj) {
    var depth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    var key = path[depth];
    var target = _typeof(obj[key]) !== "object" ? {} : obj[key];
    var subValue = depth === path.length - 1 ? value : recurse(target, depth + 1);
    return _objectSpread({}, obj, _defineProperty({}, key, subValue));
  };

  return recurse(obj);
}
function flexRender(Comp, props) {
  if (typeof Comp === "function") {
    return Object.getPrototypeOf(Comp).isReactComponent ? React__default.createElement(Comp, props) : Comp(props);
  }

  return Comp;
}
var mergeProps = function mergeProps() {
  var props = {};

  for (var _len = arguments.length, groups = new Array(_len), _key = 0; _key < _len; _key++) {
    groups[_key] = arguments[_key];
  }

  groups.forEach(function () {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        _ref$style = _ref.style,
        style = _ref$style === void 0 ? {} : _ref$style,
        className = _ref.className,
        rest = _objectWithoutProperties(_ref, ["style", "className"]);

    props = _objectSpread({}, props, rest, {
      style: _objectSpread({}, props.style || {}, style),
      className: [props.className, className].filter(Boolean).join(" ")
    });
  });
  return props;
};
var applyHooks = function applyHooks(hooks, initial) {
  for (var _len2 = arguments.length, args = new Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
    args[_key2 - 2] = arguments[_key2];
  }

  return hooks.reduce(function (prev, next) {
    return next.apply(void 0, [prev].concat(args));
  }, initial);
};
var applyPropHooks = function applyPropHooks(hooks) {
  for (var _len3 = arguments.length, args = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
    args[_key3 - 1] = arguments[_key3];
  }

  return hooks.reduce(function (prev, next) {
    return mergeProps(prev, next.apply(void 0, args));
  }, {});
};
function sum(arr) {
  return arr.reduce(function (prev, curr) {
    return prev + curr;
  }, 0);
}

function makePathArray(obj) {
  return flattenDeep(obj).join(".").replace(/\[/g, ".").replace(/\]/g, "").split(".");
}

function flattenDeep(arr) {
  var newArr = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  if (!Array.isArray(arr)) {
    newArr.push(arr);
  } else {
    for (var i = 0; i < arr.length; i += 1) {
      flattenDeep(arr[i], newArr);
    }
  }

  return newArr;
}

var defaultState = {};

var defaultReducer = function defaultReducer(old, newState, type) {
  return newState;
};

var useTableState = function useTableState() {
  var initialState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var overrides = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      _ref$reducer = _ref.reducer,
      reducer = _ref$reducer === void 0 ? defaultReducer : _ref$reducer,
      _ref$useState = _ref.useState,
      userUseState = _ref$useState === void 0 ? React.useState : _ref$useState;

  var _userUseState = userUseState(_objectSpread({}, defaultState, initialState)),
      _userUseState2 = _slicedToArray(_userUseState, 2),
      state = _userUseState2[0],
      setState = _userUseState2[1];

  var overriddenState = React.useMemo(function () {
    var newState = _objectSpread({}, state);

    Object.keys(overrides).forEach(function (key) {
      newState[key] = overrides[key];
    });
    return newState;
  }, [state].concat(_toConsumableArray(Object.values(overrides))));

  var reducedSetState = function reducedSetState(updater, type) {
    return setState(function (old) {
      var newState = updater(old);
      return reducer(old, newState, type);
    });
  };

  return [overriddenState, reducedSetState];
};

var renderErr = 'You must specify a render "type". This could be "Header", "Filter", or any other custom renderers you have set on your column.';
var propTypes = {
  // General
  data: PropTypes.any,
  columns: PropTypes.arrayOf(PropTypes.shape({
    aggregate: PropTypes.func,
    filterFn: PropTypes.func,
    filterAll: PropTypes.bool,
    sortByFn: PropTypes.func,
    resolvedDefaultSortDesc: PropTypes.bool,
    canSortBy: PropTypes.bool,
    canGroupBy: PropTypes.bool,
    Cell: PropTypes.any,
    Header: PropTypes.any,
    Filter: PropTypes.any
  })),
  filterFn: PropTypes.func,
  sortByFn: PropTypes.func,
  orderByFn: PropTypes.func,
  groupByFn: PropTypes.func,
  manualGrouping: PropTypes.bool,
  manualFilters: PropTypes.bool,
  manualSorting: PropTypes.bool,
  defaultSortDesc: PropTypes.bool,
  disableMultiSort: PropTypes.bool,
  subRowsKey: PropTypes.string,
  expandedKey: PropTypes.string,
  userAggregations: PropTypes.object,
  debug: PropTypes.bool
};
var useTable = function useTable(props) {
  // Validate props
  PropTypes.checkPropTypes(propTypes, props, 'property', 'useTable'); // Destructure props

  var _props$data = props.data,
      data = _props$data === void 0 ? [] : _props$data,
      userState = props.state,
      debug = props.debug; // Always provide a default state

  var defaultState$$1 = useTableState(); // But use the users state if provided

  var state = userState || defaultState$$1; // These are hooks that plugins can use right before render

  var hooks = {
    beforeRender: [],
    columns: [],
    headers: [],
    headerGroups: [],
    rows: [],
    row: [],
    renderableRows: [],
    getTableProps: [],
    getRowProps: [],
    getHeaderRowProps: [],
    getHeaderProps: [],
    getCellProps: [] // The initial api

  };

  var api = _objectSpread({}, props, {
    data: data,
    state: state,
    hooks: hooks
  });

  if (debug) console.time('hooks'); // Loop through plugins to build the api out

  for (var _len = arguments.length, plugins = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    plugins[_key - 1] = arguments[_key];
  }

  api = plugins.filter(Boolean).reduce(function (prev, next) {
    return next(prev);
  }, api);
  if (debug) console.timeEnd('hooks'); // Run the beforeRender hook

  if (debug) console.time('hooks.beforeRender');
  applyHooks(api.hooks.beforeRender, undefined, api);
  if (debug) console.timeEnd('hooks.beforeRender');
  if (debug) console.time('hooks.columns');
  api.columns = applyHooks(api.hooks.columns, api.columns, api);
  if (debug) console.timeEnd('hooks.columns');
  if (debug) console.time('hooks.headers');
  api.headers = applyHooks(api.hooks.headers, api.headers, api);
  if (debug) console.timeEnd('hooks.headers');
  [].concat(_toConsumableArray(api.columns), _toConsumableArray(api.headers)).forEach(function (column) {
    // Give columns/headers rendering power
    column.render = function (type) {
      var userProps = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      if (!type) {
        throw new Error(renderErr);
      }

      return flexRender(column[type], _objectSpread({}, api, column, userProps));
    }; // Give columns/headers getHeaderProps


    column.getHeaderProps = function (props) {
      return mergeProps({
        key: ['header', column.id].join('_')
      }, applyPropHooks(api.hooks.getHeaderProps, column, api), props);
    };
  });
  if (debug) console.time('hooks.headerGroups');
  api.headerGroups = applyHooks(api.hooks.headerGroups, api.headerGroups, api).filter(function (headerGroup, i) {
    // Filter out any headers and headerGroups that don't have visible columns
    headerGroup.headers = headerGroup.headers.filter(function (header) {
      var recurse = function recurse(columns) {
        return columns.filter(function (column) {
          if (column.columns) {
            return recurse(column.columns);
          }

          return column.visible;
        }).length;
      };

      if (header.columns) {
        return recurse(header.columns);
      }

      return header.visible;
    }); // Give headerGroups getRowProps

    if (headerGroup.headers.length) {
      headerGroup.getRowProps = function () {
        var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        return mergeProps({
          key: ["header".concat(i)].join('_')
        }, applyPropHooks(api.hooks.getHeaderRowProps, headerGroup, api), props);
      };

      return true;
    }

    return false;
  });
  if (debug) console.timeEnd('hooks.headerGroups'); // Run the rows (this could be a dangerous hook with a ton of data)

  if (debug) console.time('hooks.rows');
  api.rows = applyHooks(api.hooks.rows, api.rows, api);
  if (debug) console.timeEnd('hooks.rows'); // This function is absolutely necessary and MUST be called on
  // any rows the user wishes to be displayed.

  api.prepareRow = function (row) {
    var index = row.index;

    row.getRowProps = function (props) {
      return mergeProps({
        key: ['row', index].join('_')
      }, applyHooks(api.hooks.getRowProps, row, api), props);
    };

    row.cells = row.cells.filter(function (cell) {
      return cell.column.visible;
    });
    row.cells.forEach(function (cell) {
      if (!cell) {
        return;
      }

      var column = cell.column;

      cell.getCellProps = function (props) {
        var columnPathStr = [index, column.id].join('_');
        return mergeProps({
          key: ['cell', columnPathStr].join('_')
        }, applyPropHooks(api.hooks.getCellProps, cell, api), props);
      };

      cell.render = function (type) {
        var userProps = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        if (!type) {
          throw new Error('You must specify a render "type". This could be "Cell", "Header", "Filter", "Aggregated" or any other custom renderers you have set on your column.');
        }

        return flexRender(column[type], _objectSpread({}, api, cell, userProps));
      };
    });
  };

  api.getTableProps = function (userProps) {
    return mergeProps(applyPropHooks(api.hooks.getTableProps, api), userProps);
  };

  api.getRowProps = function (userProps) {
    return mergeProps(applyPropHooks(api.hooks.getRowProps, api), userProps);
  };

  return api;
};

var useColumns = function useColumns(props) {
  var debug = props.debug,
      userColumns = props.columns,
      _props$state = _slicedToArray(props.state, 1),
      groupBy = _props$state[0].groupBy;

  var _useMemo = React.useMemo(function () {
    if (debug) console.info("getColumns"); // Decorate All the columns

    var columnTree = decorateColumnTree(userColumns); // Get the flat list of all columns

    var columns = flattenBy(columnTree, "columns");
    columns = [].concat(_toConsumableArray(groupBy.map(function (g) {
      return columns.find(function (col) {
        return col.id === g;
      });
    })), _toConsumableArray(columns.filter(function (col) {
      return !groupBy.includes(col.id);
    }))); // Get headerGroups

    var headerGroups = makeHeaderGroups(columns, findMaxDepth(columnTree));
    var headers = flattenBy(headerGroups, "headers");
    return {
      columns: columns,
      headerGroups: headerGroups,
      headers: headers
    };
  }, [groupBy, userColumns]),
      columns = _useMemo.columns,
      headerGroups = _useMemo.headerGroups,
      headers = _useMemo.headers;

  return _objectSpread({}, props, {
    columns: columns,
    headerGroups: headerGroups,
    headers: headers
  }); // Find the depth of the columns

  function findMaxDepth(columns) {
    var depth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    return columns.reduce(function (prev, curr) {
      if (curr.columns) {
        return Math.max(prev, findMaxDepth(curr.columns, depth + 1));
      }

      return depth;
    }, 0);
  }

  function decorateColumn(column, parent) {
    // First check for string accessor
    var _column = column,
        id = _column.id,
        accessor = _column.accessor,
        Header = _column.Header;

    if (typeof accessor === "string") {
      id = id || accessor;
      var accessorString = accessor;

      accessor = function accessor(row) {
        return getBy(row, accessorString);
      };
    }

    if (!id && typeof Header === "string") {
      id = Header;
    }

    if (!id) {
      // Accessor, but no column id? This is bad.
      console.error(column);
      throw new Error("A column id is required!");
    }

    column = _objectSpread({
      Header: "",
      Cell: function Cell(cell) {
        return cell.value;
      },
      show: true
    }, column, {
      id: id,
      accessor: accessor,
      parent: parent
    });
    return column;
  } // Build the visible columns, headers and flat column list


  function decorateColumnTree(columns, parent) {
    var depth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
    return columns.map(function (column) {
      column = decorateColumn(column, parent);

      if (column.columns) {
        column.columns = decorateColumnTree(column.columns, column, depth + 1);
      }

      return column;
    });
  }

  function flattenBy(columns, childKey) {
    var flatColumns = [];

    var recurse = function recurse(columns) {
      columns.forEach(function (d) {
        if (!d[childKey]) {
          flatColumns.push(d);
        } else {
          recurse(d[childKey]);
        }
      });
    };

    recurse(columns);
    return flatColumns;
  } // Build the header groups from the bottom up


  function makeHeaderGroups(columns, maxDepth) {
    var headerGroups = [];

    var removeChildColumns = function removeChildColumns(column) {
      delete column.columns;

      if (column.parent) {
        removeChildColumns(column.parent);
      }
    };

    columns.forEach(removeChildColumns);

    var buildGroup = function buildGroup(columns) {
      var depth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var headerGroup = {
        headers: []
      };
      var parentColumns = [];
      var hasParents = columns.some(function (col) {
        return col.parent;
      });
      columns.forEach(function (column) {
        var isFirst = !parentColumns.length;
        var latestParentColumn = [].concat(parentColumns).reverse()[0]; // If the column has a parent, add it if necessary

        if (column.parent) {
          if (isFirst || latestParentColumn.originalID !== column.parent.id) {
            parentColumns.push(_objectSpread({}, column.parent, {
              originalID: column.parent.id,
              id: [column.parent.id, parentColumns.length].join("_")
            }));
          }
        } else if (hasParents) {
          // If other columns have parents, add a place holder if necessary
          var placeholderColumn = decorateColumn({
            originalID: [column.id, "placeholder", maxDepth - depth].join("_"),
            id: [column.id, "placeholder", maxDepth - depth, parentColumns.length].join("_")
          });

          if (isFirst || latestParentColumn.originalID !== placeholderColumn.originalID) {
            parentColumns.push(placeholderColumn);
          }
        } // Establish the new columns[] relationship on the parent


        if (column.parent || hasParents) {
          latestParentColumn = [].concat(parentColumns).reverse()[0];
          latestParentColumn.columns = latestParentColumn.columns || [];

          if (!latestParentColumn.columns.includes(column)) {
            latestParentColumn.columns.push(column);
          }
        }

        headerGroup.headers.push(column);
      });
      headerGroups.push(headerGroup);

      if (parentColumns.length) {
        buildGroup(parentColumns);
      }
    };

    buildGroup(columns);
    return headerGroups.reverse();
  }
};

var useRows = function useRows(props) {
  var debug = props.debug,
      columns = props.columns,
      _props$subRowsKey = props.subRowsKey,
      subRowsKey = _props$subRowsKey === void 0 ? "subRows" : _props$subRowsKey,
      data = props.data;
  var accessedRows = React.useMemo(function () {
    if (debug) console.info("getAccessedRows"); // Access the row's data

    var accessRow = function accessRow(originalRow, i) {
      var depth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      // Keep the original reference around
      var original = originalRow; // Process any subRows

      var subRows = originalRow[subRowsKey] ? originalRow[subRowsKey].map(function (d, i) {
        return accessRow(d, i, depth + 1);
      }) : undefined;
      var row = {
        original: original,
        index: i,
        subRows: subRows,
        depth: depth
      }; // Create the cells and values

      row.values = {};
      columns.forEach(function (column) {
        row.values[column.id] = column.accessor ? column.accessor(originalRow, i, {
          subRows: subRows,
          depth: depth,
          data: data
        }) : undefined;
      });
      return row;
    }; // Use the resolved data


    return data.map(function (d, i) {
      return accessRow(d, i);
    });
  }, [data, columns]);
  return _objectSpread({}, props, {
    rows: accessedRows
  });
};

var actions = {};
var addActions = function addActions(acts) {
  Object.keys(acts).forEach(function (key) {
    actions[key] = acts[key];
  });
};

defaultState.expanded = {};
addActions({
  toggleExpanded: "__toggleExpanded__",
  useExpanded: "__useExpanded__"
});
var useExpanded = function useExpanded(props) {
  var debug = props.debug,
      columns = props.columns,
      rows = props.rows,
      _props$expandedKey = props.expandedKey,
      expandedKey = _props$expandedKey === void 0 ? "expanded" : _props$expandedKey,
      hooks = props.hooks,
      _props$state = _slicedToArray(props.state, 2),
      expanded = _props$state[0].expanded,
      setState = _props$state[1];

  var toggleExpandedByPath = function toggleExpandedByPath(path, set) {
    return setState(function (old) {
      var expanded = old.expanded;
      var existing = getBy(expanded, path);
      set = getFirstDefined(set, !existing);
      return _objectSpread({}, old, {
        expanded: setBy(expanded, path, set)
      });
    }, actions.toggleExpanded);
  };

  hooks.row.push(function (row) {
    var path = row.path;

    row.toggleExpanded = function (set) {
      return toggleExpandedByPath(path, set);
    };
  });
  var expandedRows = React.useMemo(function () {
    if (debug) console.info("getExpandedRows");
    var expandedRows = []; // Here we do some mutation, but it's the last stage in the
    // immutable process so this is safe

    var handleRow = function handleRow(row, index) {
      var depth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var parentPath = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
      // Compute some final state for the row
      var path = [].concat(_toConsumableArray(parentPath), [index]);
      row.path = path;
      row.depth = depth;
      row.isExpanded = row.original && row.original[expandedKey] || getBy(expanded, path);
      row.cells = columns.map(function (column) {
        var cell = {
          column: column,
          row: row,
          state: null,
          value: row.values[column.id]
        };
        return cell;
      });
      expandedRows.push(row);

      if (row.isExpanded && row.subRows && row.subRows.length) {
        row.subRows.forEach(function (row, i) {
          return handleRow(row, i, depth + 1, path);
        });
      }
    };

    rows.forEach(function (row, i) {
      return handleRow(row, i);
    });
    return expandedRows;
  }, [rows, expanded, columns]);
  var expandedDepth = findExpandedDepth(expanded);
  return _objectSpread({}, props, {
    toggleExpandedByPath: toggleExpandedByPath,
    expandedDepth: expandedDepth,
    rows: expandedRows
  });
};

function findExpandedDepth(obj) {
  var depth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
  return Object.values(obj).reduce(function (prev, curr) {
    if (_typeof(curr) === "object") {
      return Math.max(prev, findExpandedDepth(curr, depth + 1));
    }

    return depth;
  }, 0);
}

defaultState.filters = {};
addActions({
  setFilter: "__setFilter__",
  setAllFilters: "__setAllFilters__"
});
var useFilters = function useFilters(props) {
  var debug = props.debug,
      rows = props.rows,
      columns = props.columns,
      _props$filterFn = props.filterFn,
      filterFn = _props$filterFn === void 0 ? defaultFilterFn : _props$filterFn,
      manualFilters = props.manualFilters,
      disableFilters = props.disableFilters,
      hooks = props.hooks,
      _props$state = _slicedToArray(props.state, 2),
      filters = _props$state[0].filters,
      setState = _props$state[1];

  columns.forEach(function (column) {
    var id = column.id,
        accessor = column.accessor,
        canFilter = column.canFilter;
    column.canFilter = accessor ? getFirstDefined(canFilter, disableFilters === true ? false : undefined, true) : false; // Was going to add this to the filter hook

    column.filterValue = filters[id];
  });

  var setFilter = function setFilter(id, val) {
    return setState(function (old) {
      if (typeof val === "undefined") {
        var prev = filters[id],
            rest = _objectWithoutProperties(filters, [id].map(_toPropertyKey));

        return _objectSpread({}, old, {
          filters: _objectSpread({}, rest)
        });
      }

      return _objectSpread({}, old, {
        filters: _objectSpread({}, filters, _defineProperty({}, id, val))
      });
    }, actions.setFilter);
  };

  var setAllFilters = function setAllFilters(filters) {
    return setState(function (old) {
      return _objectSpread({}, old, {
        filters: filters
      });
    }, actions.setAllFilters);
  };

  hooks.columns.push(function (columns) {
    columns.forEach(function (column) {
      if (column.canFilter) {
        column.setFilter = function (val) {
          return setFilter(column.id, val);
        };
      }
    });
    return columns;
  });
  var filteredRows = React.useMemo(function () {
    if (manualFilters || !Object.keys(filters).length) {
      return rows;
    }

    if (debug) console.info("getFilteredRows"); // Filters top level and nested rows

    var filterRows = function filterRows(rows) {
      var filteredRows = rows;
      filteredRows = Object.entries(filters).reduce(function (filteredSoFar, _ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            columnID = _ref2[0],
            filterValue = _ref2[1];

        // Find the filters column
        var column = columns.find(function (d) {
          return d.id === columnID;
        }); // Don't filter hidden columns or columns that have had their filters disabled

        if (!column || column.filterable === false) {
          return filteredSoFar;
        }

        var filterMethod = column.filterMethod || filterFn; // If 'filterAll' is set to true, pass the entire dataset to the filter method

        if (column.filterAll) {
          return filterMethod(filteredSoFar, columnID, filterValue, column);
        }

        return filteredSoFar.filter(function (row) {
          return filterMethod(row, columnID, filterValue, column);
        });
      }, rows); // Apply the filter to any subRows

      filteredRows = filteredRows.map(function (row) {
        if (!row.subRows) {
          return row;
        }

        return _objectSpread({}, row, {
          subRows: filterRows(row.subRows)
        });
      }); // then filter any rows without subcolumns because it would be strange to show

      filteredRows = filteredRows.filter(function (row) {
        if (!row.subRows) {
          return true;
        }

        return row.subRows.length > 0;
      });
      return filteredRows;
    };

    return filterRows(rows);
  }, [rows, filters, manualFilters]);
  return _objectSpread({}, props, {
    setFilter: setFilter,
    setAllFilters: setAllFilters,
    rows: filteredRows
  });
};

function sum$1(values, rows) {
  return values.reduce(function (sum, next) {
    return sum + next;
  }, 0);
}
function average(values, rows) {
  return Math.round(sum$1(values, rows) / values.length * 100) / 100;
}

var aggregations = /*#__PURE__*/Object.freeze({
  sum: sum$1,
  average: average
});

defaultState.groupBy = [];
addActions({
  toggleGroupBy: "__toggleGroupBy__"
});
var useGroupBy = function useGroupBy(api) {

  var debug = api.debug,
      rows = api.rows,
      columns = api.columns,
      _api$groupByFn = api.groupByFn,
      groupByFn = _api$groupByFn === void 0 ? defaultGroupByFn : _api$groupByFn,
      manualGroupBy = api.manualGroupBy,
      disableGrouping = api.disableGrouping,
      _api$aggregations = api.aggregations,
      userAggregations = _api$aggregations === void 0 ? {} : _api$aggregations,
      hooks = api.hooks,
      _api$state = _slicedToArray(api.state, 2),
      groupBy = _api$state[0].groupBy,
      setState = _api$state[1];

  columns.forEach(function (column) {
    var id = column.id,
        accessor = column.accessor,
        canGroupBy = column.canGroupBy;
    column.grouped = groupBy.includes(id);
    column.canGroupBy = accessor ? getFirstDefined(canGroupBy, disableGrouping === true ? false : undefined, true) : false;
    column.Aggregated = column.Aggregated || column.Cell;
  });

  var toggleGroupBy = function toggleGroupBy(id, toggle) {
    return setState(function (old) {
      var resolvedToggle = typeof set !== "undefined" ? toggle : !groupBy.includes(id);

      if (resolvedToggle) {
        return _objectSpread({}, old, {
          groupBy: [].concat(_toConsumableArray(groupBy), [id])
        });
      }

      return _objectSpread({}, old, {
        groupBy: groupBy.filter(function (d) {
          return d !== id;
        })
      });
    }, actions.toggleGroupBy);
  };

  hooks.columns.push(function (columns) {
    columns.forEach(function (column) {
      if (column.canGroupBy) {
        column.toggleGroupBy = function () {
          return toggleGroupBy(column.id);
        };
      }
    });
    return columns;
  });
  hooks.getGroupByToggleProps = [];

  var addGroupByToggleProps = function addGroupByToggleProps(columns, api) {
    columns.forEach(function (column) {
      var canGroupBy = column.canGroupBy;

      column.getGroupByToggleProps = function (props) {
        return mergeProps({
          onClick: canGroupBy ? function (e) {
            e.persist();
            column.toggleGroupBy();
          } : undefined,
          style: {
            cursor: canGroupBy ? "pointer" : undefined
          },
          title: "Toggle GroupBy"
        }, applyPropHooks(api.hooks.getGroupByToggleProps, column, api), props);
      };
    });
    return columns;
  };

  hooks.columns.push(addGroupByToggleProps);
  hooks.headers.push(addGroupByToggleProps);
  var groupedRows = React.useMemo(function () {
    if (manualGroupBy || !groupBy.length) {
      return rows;
    }

    if (debug) console.info("getGroupedRows"); // Find the columns that can or are aggregating
    // Uses each column to aggregate rows into a single value

    var aggregateRowsToValues = function aggregateRowsToValues(rows) {
      var values = {};
      columns.forEach(function (column) {
        var columnValues = rows.map(function (d) {
          return d.values[column.id];
        });
        var aggregate = userAggregations[column.aggregate] || aggregations[column.aggregate] || column.aggregate;

        if (typeof aggregate === "function") {
          values[column.id] = aggregate(columnValues, rows);
        } else if (aggregate) {
          throw new Error("Invalid aggregate \"".concat(aggregate, "\" passed to column with ID: \"").concat(column.id, "\""));
        } else {
          values[column.id] = columnValues[0];
        }
      });
      return values;
    }; // Recursively group the data


    var groupRecursively = function groupRecursively(rows, groupBy) {
      var depth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

      // This is the last level, just return the rows
      if (depth >= groupBy.length) {
        return rows;
      } // Group the rows together for this level


      var groupedRows = Object.entries(groupByFn(rows, groupBy[depth])).map(function (_ref, index) {
        var _ref2 = _slicedToArray(_ref, 2),
            groupByVal = _ref2[0],
            subRows = _ref2[1];

        // Recurse to sub rows before aggregation
        subRows = groupRecursively(subRows, groupBy, depth + 1);
        var values = aggregateRowsToValues(subRows);
        var row = {
          groupByID: groupBy[depth],
          groupByVal: groupByVal,
          values: values,
          subRows: subRows,
          depth: depth,
          index: index
        };
        return row;
      });
      return groupedRows;
    }; // Assign the new data


    return groupRecursively(rows, groupBy);
  }, [rows, groupBy, columns, manualGroupBy]);
  return _objectSpread({}, api, {
    rows: groupedRows
  });
};

defaultState.sortBy = [];
addActions({
  sortByChange: "__sortByChange__"
});
var useSortBy = function useSortBy(api) {
  var debug = api.debug,
      rows = api.rows,
      columns = api.columns,
      _api$orderByFn = api.orderByFn,
      orderByFn = _api$orderByFn === void 0 ? defaultOrderByFn : _api$orderByFn,
      _api$sortByFn = api.sortByFn,
      sortByFn = _api$sortByFn === void 0 ? defaultSortByFn : _api$sortByFn,
      manualSorting = api.manualSorting,
      disableSorting = api.disableSorting,
      defaultSortDesc = api.defaultSortDesc,
      hooks = api.hooks,
      _api$state = _slicedToArray(api.state, 2),
      sortBy = _api$state[0].sortBy,
      setState = _api$state[1];

  columns.forEach(function (column) {
    var accessor = column.accessor,
        canSortBy = column.canSortBy;
    column.canSortBy = accessor ? getFirstDefined(canSortBy, disableSorting === true ? false : undefined, true) : false;
  }); // Updates sorting based on a columnID, desc flag and multi flag

  var toggleSortByID = function toggleSortByID(columnID, desc, multi) {
    return setState(function (old) {
      var sortBy = old.sortBy; // Find the column for this columnID

      var column = columns.find(function (d) {
        return d.id === columnID;
      });
      var resolvedDefaultSortDesc = getFirstDefined(column.defaultSortDesc, defaultSortDesc); // Find any existing sortBy for this column

      var existingSortBy = sortBy.find(function (d) {
        return d.id === columnID;
      });
      var hasDescDefined = typeof desc !== "undefined" && desc !== null;
      var newSortBy = []; // What should we do with this filter?

      var action;

      if (!multi) {
        if (sortBy.length <= 1 && existingSortBy) {
          if (existingSortBy.desc) {
            action = "remove";
          } else {
            action = "toggle";
          }
        } else {
          action = "replace";
        }
      } else {
        if (!existingSortBy) {
          action = "add";
        } else {
          if (hasDescDefined) {
            action = "set";
          } else {
            action = "toggle";
          }
        }
      }

      if (action === "replace") {
        newSortBy = [{
          id: columnID,
          desc: hasDescDefined ? desc : resolvedDefaultSortDesc
        }];
      } else if (action === "add") {
        newSortBy = [].concat(_toConsumableArray(sortBy), [{
          id: columnID,
          desc: hasDescDefined ? desc : resolvedDefaultSortDesc
        }]);
      } else if (action === "set") {
        newSortBy = sortBy.map(function (d) {
          if (d.id === columnID) {
            return _objectSpread({}, d, {
              desc: desc
            });
          }

          return d;
        });
      } else if (action === "toggle") {
        newSortBy = sortBy.map(function (d) {
          if (d.id === columnID) {
            return _objectSpread({}, d, {
              desc: !existingSortBy.desc
            });
          }

          return d;
        });
      } else if (action === "remove") {
        newSortBy = [];
      }

      return _objectSpread({}, old, {
        sortBy: newSortBy
      });
    }, actions.sortByChange);
  };

  hooks.columns.push(function (columns) {
    columns.forEach(function (column) {
      if (column.canSortBy) {
        column.toggleSortBy = function (desc, multi) {
          return toggleSortByID(column.id, desc, multi);
        };
      }
    });
    return columns;
  });
  hooks.getSortByToggleProps = [];

  var addSortByToggleProps = function addSortByToggleProps(columns, api) {
    columns.forEach(function (column) {
      var canSortBy = column.canSortBy;

      column.getSortByToggleProps = function (props) {
        return mergeProps({
          onClick: canSortBy ? function (e) {
            e.persist();
            column.toggleSortBy(undefined, !api.disableMultiSort && e.shiftKey);
          } : undefined,
          style: {
            cursor: canSortBy ? "pointer" : undefined
          },
          title: "Toggle SortBy"
        }, applyPropHooks(api.hooks.getSortByToggleProps, column, api), props);
      };
    });
    return columns;
  };

  hooks.columns.push(addSortByToggleProps);
  hooks.headers.push(addSortByToggleProps); // Mutate columns to reflect sorting state

  columns.forEach(function (column) {
    var id = column.id;
    column.sorted = sortBy.find(function (d) {
      return d.id === id;
    });
    column.sortedIndex = sortBy.findIndex(function (d) {
      return d.id === id;
    });
    column.sortedDesc = column.sorted ? column.sorted.desc : undefined;
  });
  var sortedRows = React.useMemo(function () {
    if (manualSorting || !sortBy.length) {
      return rows;
    }

    if (debug) console.info("getSortedRows");
    var sortMethodsByColumnID = {};
    columns.filter(function (col) {
      return col.sortMethod;
    }).forEach(function (col) {
      sortMethodsByColumnID[col.id] = col.sortMethod;
    });

    var sortData = function sortData(rows) {
      // Use the orderByFn to compose multiple sortBy's together.
      // This will also perform a stable sorting using the row index
      // if needed.
      var sortedData = orderByFn(rows, sortBy.map(function (sort) {
        // Support custom sorting methods for each column
        var columnSortBy = sortMethodsByColumnID[sort.id]; // Return the correct sortFn

        return function (a, b) {
          return (columnSortBy ? columnSortBy : sortByFn)(a.values[sort.id], b.values[sort.id], sort.desc);
        };
      }), // Map the directions
      sortBy.map(function (d) {
        return !d.desc;
      })); // TODO: this should be optimized. Not good to loop again

      sortedData.forEach(function (row) {
        if (!row.subRows) {
          return;
        }

        row.subRows = sortData(row.subRows);
      });
      return sortedData;
    };

    return sortData(rows);
  }, [rows, columns, sortBy, manualSorting]);
  return _objectSpread({}, api, {
    rows: sortedRows
  });
};

defaultState.pageSize = 10;
defaultState.pageIndex = 0;
addActions({
  pageChange: "__pageChange__"
});
var propTypes$1 = {
  defaultPageSize: PropTypes.number,
  defaultPageIndex: PropTypes.number,
  pageSize: PropTypes.number,
  pages: PropTypes.number,
  pageIndex: PropTypes.number,
  onStateChange: PropTypes.func,
  stateReducer: PropTypes.func,
  debug: PropTypes.bool
};
var usePagination = function usePagination(props) {
  // Validate props
  PropTypes.checkPropTypes(propTypes$1, props, "property", "usePagination");

  var parentDebug = props.debug,
      rows = props.rows,
      manualPagination = props.manualPagination,
      _props$debug = props.debug,
      debug = _props$debug === void 0 ? parentDebug : _props$debug,
      _props$state = _slicedToArray(props.state, 2),
      _props$state$ = _props$state[0],
      pageSize = _props$state$.pageSize,
      pageIndex = _props$state$.pageIndex,
      userPageCount = _props$state$.pageCount,
      filters = _props$state$.filters,
      groupBy = _props$state$.groupBy,
      sortBy = _props$state$.sortBy,
      setState = _props$state[1];

  React.useLayoutEffect(function () {
    setState(function (old) {
      return _objectSpread({}, old, {
        pageIndex: 0
      });
    }, actions.pageChange);
  }, [filters, groupBy, sortBy]);

  var _useMemo = React.useMemo(function () {
    if (manualPagination) {
      return {
        pages: [rows],
        pageCount: userPageCount
      };
    }

    if (debug) console.info("getPages"); // Create a new pages with the first page ready to go.

    var pages = rows.length ? [] : [[]]; // Start the pageIndex and currentPage cursors

    var cursor = 0;

    while (cursor < rows.length) {
      var end = cursor + pageSize;
      pages.push(rows.slice(cursor, end));
      cursor = end;
    }

    var pageCount = pages.length;
    return {
      pages: pages,
      pageCount: pageCount,
      pageOptions: pageOptions
    };
  }, [rows, pageSize, userPageCount]),
      pages = _useMemo.pages,
      pageCount = _useMemo.pageCount;

  var pageOptions = _toConsumableArray(new Array(pageCount)).map(function (d, i) {
    return i;
  });

  var page = manualPagination ? rows : pages[pageIndex] || [];
  var canPreviousPage = pageIndex > 0;
  var canNextPage = pageIndex < pageCount - 1;

  var gotoPage = function gotoPage(pageIndex) {
    if (debug) console.info("gotoPage");
    return setState(function (old) {
      if (pageIndex < 0 || pageIndex > pageCount - 1) {
        return old;
      }

      return _objectSpread({}, old, {
        pageIndex: pageIndex
      });
    }, actions.pageChange);
  };

  var previousPage = function previousPage() {
    return gotoPage(pageIndex - 1);
  };

  var nextPage = function nextPage() {
    return gotoPage(pageIndex + 1);
  };

  var setPageSize = function setPageSize(pageSize) {
    setState(function (old) {
      var topRowIndex = old.pageSize * old.pageIndex;
      var pageIndex = Math.floor(topRowIndex / pageSize);
      return _objectSpread({}, old, {
        pageIndex: pageIndex,
        pageSize: pageSize
      });
    }, actions.setPageSize);
  };

  return _objectSpread({}, props, {
    pages: pages,
    pageOptions: pageOptions,
    page: page,
    canPreviousPage: canPreviousPage,
    canNextPage: canNextPage,
    gotoPage: gotoPage,
    previousPage: previousPage,
    nextPage: nextPage,
    setPageSize: setPageSize
  });
};

var propTypes$2 = {
  defaultFlex: PropTypes.number
};
var useFlexLayout = function useFlexLayout(props) {
  // Validate props
  PropTypes.checkPropTypes(propTypes$2, props, "property", "useFlexLayout");
  var _props$defaultFlex = props.defaultFlex,
      defaultFlex = _props$defaultFlex === void 0 ? 1 : _props$defaultFlex,
      _props$hooks = props.hooks,
      columnsHooks = _props$hooks.columns,
      getRowProps = _props$hooks.getRowProps,
      getHeaderRowProps = _props$hooks.getHeaderRowProps,
      getHeaderProps = _props$hooks.getHeaderProps,
      getCellProps = _props$hooks.getCellProps;
  columnsHooks.push(function (columns, api) {
    var visibleColumns = columns.filter(function (column) {
      column.visible = typeof column.show === "function" ? column.show(api) : !!column.show;
      return column.visible;
    });
    var columnMeasurements = {};
    var sumWidth = 0;
    visibleColumns.forEach(function (column) {
      var _getSizesForColumn = getSizesForColumn(column, defaultFlex, undefined, undefined, api),
          width = _getSizesForColumn.width,
          minWidth = _getSizesForColumn.minWidth;

      if (width) {
        sumWidth += width;
      } else if (minWidth) {
        sumWidth += minWidth;
      } else {
        sumWidth += defaultFlex;
      }
    });
    var rowStyles = {
      style: {
        display: "flex",
        minWidth: "".concat(sumWidth, "px")
      }
    };
    api.rowStyles = rowStyles;
    getRowProps.push(function () {
      return rowStyles;
    });
    getHeaderRowProps.push(function () {
      return rowStyles;
    });
    getHeaderProps.push(function (column) {
      return {
        style: _objectSpread({
          boxSizing: "border-box"
        }, getStylesForColumn(column, columnMeasurements, defaultFlex, api)) // [refKey]: el => {
        //   renderedCellInfoRef.current[key] = {
        //     column,
        //     el
        //   };
        // },

      };
    });
    getCellProps.push(function (cell) {
      return {
        style: _objectSpread({
          display: "block",
          boxSizing: "border-box"
        }, getStylesForColumn(cell.column, columnMeasurements, defaultFlex, undefined, api)) // [refKey]: el => {
        //   renderedCellInfoRef.current[columnPathStr] = {
        //     column,
        //     el
        //   };
        // }

      };
    });
    return columns;
  });
  return props;
}; // Utils

function getStylesForColumn(column, columnMeasurements, defaultFlex, api) {
  var _getSizesForColumn2 = getSizesForColumn(column, columnMeasurements, defaultFlex, api),
      flex = _getSizesForColumn2.flex,
      width = _getSizesForColumn2.width,
      maxWidth = _getSizesForColumn2.maxWidth;

  return {
    flex: "".concat(flex, " 0 auto"),
    width: "".concat(width, "px"),
    maxWidth: "".concat(maxWidth, "px")
  };
}

function getSizesForColumn(_ref, columnMeasurements, defaultFlex, api) {
  var columns = _ref.columns,
      id = _ref.id,
      width = _ref.width,
      minWidth = _ref.minWidth,
      maxWidth = _ref.maxWidth;

  if (columns) {
    columns = columns.map(function (column) {
      return getSizesForColumn(column, columnMeasurements, defaultFlex, api);
    }).filter(Boolean);

    if (!columns.length) {
      return false;
    }

    var flex = sum(columns.map(function (col) {
      return col.flex;
    }));

    var _width = sum(columns.map(function (col) {
      return col.width;
    }));

    var _maxWidth = sum(columns.map(function (col) {
      return col.maxWidth;
    }));

    return {
      flex: flex,
      width: _width,
      maxWidth: _maxWidth
    };
  }

  return {
    flex: width ? 0 : defaultFlex,
    width: width === "auto" ? columnMeasurements[id] || defaultFlex : getFirstDefined(width, minWidth, defaultFlex),
    maxWidth: maxWidth
  };
} // const resetRefs = () => {
//   if (debug) console.info("resetRefs");
//   renderedCellInfoRef.current = {};
// };
// const calculateAutoWidths = () => {
//   RAF(() => {
//     const newColumnMeasurements = {};
//     Object.values(renderedCellInfoRef.current).forEach(({ column, el }) => {
//       if (!el) {
//         return;
//       }
//       let measurement = 0;
//       const measureChildren = children => {
//         if (children) {
//           [].slice.call(children).forEach(child => {
//             measurement = Math.max(
//               measurement,
//               Math.ceil(child.offsetWidth) || 0
//             );
//             measureChildren(child.children);
//           });
//         }
//         return measurement;
//       };
//       const parentDims = getElementDimensions(el);
//       measureChildren(el.children);
//       newColumnMeasurements[column.id] = Math.max(
//         newColumnMeasurements[column.id] || 0,
//         measurement + parentDims.paddingLeft + parentDims.paddingRight
//       );
//     });
//     const oldKeys = Object.keys(columnMeasurements);
//     const newKeys = Object.keys(newColumnMeasurements);
//     const needsUpdate =
//       oldKeys.length !== newKeys.length ||
//       oldKeys.some(key => {
//         return columnMeasurements[key] !== newColumnMeasurements[key];
//       });
//     if (needsUpdate) {
//       setState(old => {
//         return {
//           ...old,
//           columnMeasurements: newColumnMeasurements
//         };
//       }, actions.updateAutoWidth);
//     }
//   });
// };

exports.useTable = useTable;
exports.useColumns = useColumns;
exports.useRows = useRows;
exports.useExpanded = useExpanded;
exports.useFilters = useFilters;
exports.useGroupBy = useGroupBy;
exports.useSortBy = useSortBy;
exports.usePagination = usePagination;
exports.useTableState = useTableState;
exports.useFlexLayout = useFlexLayout;
exports.actions = actions;
//# sourceMappingURL=index.js.map
